import json
import os
import webbrowser
from typing import Any

import fire
import tweepy

BASE_URL = "https://api.twitter.com/"
ROOT_PATH = os.environ["HOME"] + "/.local/clitw"
TOKEN_FILE = ROOT_PATH + "/oauth_token.json"
CONSUMER_FILE = ROOT_PATH + "/consumer_token.json"


def make_auth() -> Any:
    with open(CONSUMER_FILE, mode="+r") as f:
        consumer = json.load(f)
    return tweepy.OAuthHandler(**consumer)


def authenicate_oauth(auth: Any) -> str:
    try:
        redirect_url = auth.get_authorization_url()
    except Exception as e:
        raise tweepy.TweepError(e)

    if webbrowser.open_new_tab(redirect_url):
        return input("Verifier: ")
    else:
        raise ValueError()


def load_access_token() -> Any:
    auth = make_auth()
    if os.path.isfile(TOKEN_FILE):
        with open(TOKEN_FILE, "r") as f:
            token = json.load(f)
    else:
        verifier = authenicate_oauth(auth)
        try:
            key, secret = auth.get_access_token(verifier)
        except Exception as e:
            raise tweepy.TweepError(e)

        token = {"key": key, "secret": secret}
        with open(TOKEN_FILE, mode="w") as f:
            json.dump(token, f)
    auth.set_access_token(**token)
    return auth


def make_api() -> Any:
    auth = load_access_token()
    return tweepy.API(auth)


class Twitter:
    def __init__(self) -> None:
        if not os.path.isdir(ROOT_PATH):
            os.makedirs(ROOT_PATH)
        self.api = make_api()

    def tw(self) -> None:
        flag = "n"
        text = ""
        while flag != "y":
            text = input("Text: ")
            print(text)
            flag = input("Tweet? (y or n): ")
        try:
            self.api.update_status(text)
        except Exception as e:
            raise tweepy.TweepError(e)
        print("success")


def main() -> None:
    fire.Fire(Twitter)



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

modules = \
['clitw']
install_requires = \
['fire>=0.4.0,<0.5.0', 'tweepy>=3.10.0,<4.0.0']

entry_points = \
{'console_scripts': ['clitw = clitw:main']}

setup_kwargs = {
    'name': 'clitw',
    'version': '0.0.10',
    'description': 'CUI Twitter Client on Python',
    'long_description': 'tamatebako\n==========\n\n::\n\n   pip install tamatebako\n   tw\n',
    'author': 'Daisuke Oku',
    'author_email': 'w.40141@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/w40141',
    'package_dir': package_dir,
    'py_modules': modules,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
